"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createResource = void 0;
var warning_1 = __importDefault(require("warning"));
var vue_demi_1 = require("vue-demi");
var Suspense_1 = require("./Suspense");
var suspenseInstance_1 = require("../shared/suspenseInstance");
function createResource(fetchFactory, options) {
    var $res = (0, vue_demi_1.reactive)({
        $$result: void 0,
        $$error: void 0,
        $$loading: false,
        $$loaded: false, // it has loaded data successed once more
    });
    var hasSuspenseInstance = false;
    // 在组件内部创建, 查找Suspense实例
    var currInstance = (0, vue_demi_1.getCurrentInstance)();
    if ((options === null || options === void 0 ? void 0 : options.suspensible) !== false && (currInstance === null || currInstance === void 0 ? void 0 : currInstance.proxy)) {
        fetchFactory.suspenseInstance = (0, suspenseInstance_1.findSuspenseInstance)(currInstance.proxy);
        hasSuspenseInstance = !!fetchFactory.suspenseInstance;
    }
    var resourceResult = {
        read: function () {
            var input = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                input[_i] = arguments[_i];
            }
            // prevent
            if (options && options.prevent && $res.$$loading) {
                return $res.$$promiser;
            }
            $res.$$loading = true;
            // Because we don't need caching, this is just a unique identifier,
            // and each call to .read() is a completely new request.
            var uniqueWrapFactory = function () {
                var i = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    i[_i] = arguments[_i];
                }
                return fetchFactory.apply(void 0, __spreadArray([], __read(i), false));
            };
            if (hasSuspenseInstance) {
                // Establish a relationship between the fetchFactory and the current component instance
                uniqueWrapFactory.suspenseInstance = fetchFactory.suspenseInstance;
                (0, Suspense_1.add)(uniqueWrapFactory);
            }
            // Start fetching asynchronous data
            var promise = ($res.$$promiser = uniqueWrapFactory.apply(void 0, __spreadArray([], __read(input), false)));
            promise
                .then(function (res) {
                // Trigger update
                $res.$$result = (options === null || options === void 0 ? void 0 : options.onSuccess) ? options.onSuccess(res) : res;
                if (!$res.$$loaded) {
                    $res.$$loaded = true;
                }
                if (hasSuspenseInstance) {
                    (0, Suspense_1.del)(uniqueWrapFactory);
                }
            })
                .catch(function (err) {
                (0, warning_1.default)(process.env.NODE_ENV === 'production', err.message);
                $res.$$error = (options === null || options === void 0 ? void 0 : options.onError) ? options.onError(err) : err;
                if (hasSuspenseInstance) {
                    (0, Suspense_1.del)(uniqueWrapFactory, err);
                }
            })
                .finally(function () {
                $res.$$loading = false;
            });
            return promise;
        },
        get $result() {
            return $res.$$result;
        },
        set $result(val) {
            $res.$$result = val;
        },
        get $error() {
            return $res.$$error;
        },
        set $error(val) {
            $res.$$error = val;
        },
        get $loading() {
            return $res.$$loading;
        },
        get $loaded() {
            return $res.$$loaded;
        },
        fork: function () {
            return createResource(function () {
                var i = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    i[_i] = arguments[_i];
                }
                return fetchFactory.apply(void 0, __spreadArray([], __read(i), false));
            }, options);
        },
    };
    return resourceResult;
}
exports.createResource = createResource;
//# sourceMappingURL=createResource.js.map